# encoding: utf-8
# WARNING! Do not edit! https://waf.io/book/index.html#_obtaining_the_waf_file

import contextlib
import functools
import glob
import inspect
import json
import logging
import os
import re
import subprocess
import tempfile
try:
	from waflib import Configure,Context,Options
except ImportError:
	pass
@functools.total_ordering
class _VisualStudioToolset(object):
	VS_INSTALL_ROOT='{}\\Microsoft Visual Studio'.format(os.environ.get('PROGRAMFILES(X86)')or os.environ.get('PROGRAMFILES')or'C:\\Program Files (x86)',)
	_discovered=None
	@classmethod
	def discover(cls,log_debug,runner=None):
		if cls._discovered is None:
			run_process=runner or functools.partial(subprocess.check_output,universal_newlines=True,)
			listing=run_process(['{}/Installer/vswhere.exe'.format(cls.VS_INSTALL_ROOT),'-all','-products','*','-format','json','-utf8','-include','packages',],)
			installations=json.loads(listing)
			vc_components='{}(?:{})'.format(re.escape('Microsoft.VisualStudio.Component.VC.'),'|'.join([r'Tools\.(?!14)','140','v141',r'(?:Tools\.)?14(?:\.\d+)+',],),)
			re_sxs_ids=re.compile(vc_components)
			vc_components=('{}:{}'.format(install['installationVersion'],package['id'])for install in installations for package in install['packages']if re_sxs_ids.match(package['id']))
			for component in vc_components:
				log_debug(component)
			sxs_minimum_version=15
			installs=((install['installationVersion'],install['installationPath'])for install in installations if next(int(i)for i in install['installationVersion'].split('.'))>=sxs_minimum_version)
			toolset_dirs={cls(install_version,toolset_dir)for install_version,install_path in installs for toolset_dir in glob.iglob('{}/VC/Tools/MSVC/*'.format(install_path))if os.path.isdir('{}/bin'.format(toolset_dir))}
			cls._discovered=sorted(toolset_dirs)
		return cls._discovered
	def __init__(self,install_version,directory):
		self.install_version=install_version
		self._path=directory
		self._vcvars_ver=os.path.basename(directory)
		self._vcvarsall=os.path.normpath('{}/../../../Auxiliary/Build/vcvarsall.bat'.format(directory),)
		version=tuple(int(i)for i in self._vcvars_ver.split('.'))
		if version<(14,20):
			self._visual_studio_version=(15,version[1]-7)
		elif version<(14,30):
			self._visual_studio_version=(16,version[1]-20+bool(version>=(14,28,29910))+bool(version>=(14,29,30129)),)
		else:
			self._visual_studio_version=(17,version[1]-30)
	def __eq__(self,other):
		return self._visual_studio_version==other._visual_studio_version
	def __lt__(self,other):
		return self._visual_studio_version<other._visual_studio_version
	def __hash__(self):
		return hash(self._path)
	def __str__(self):
		return'{:<16s} {:<6s} {}'.format('{}:'.format(self.install_version),'{}:'.format(self.visual_studio_version),self.vc_version,)
	@property
	def visual_studio_version(self):
		return'{}.{}'.format(*self._visual_studio_version)
	@property
	def vcvarsall(self):
		return self._vcvarsall
	@property
	def vc_version(self):
		return self._vcvars_ver
	def cl(self):
		contents=inspect.cleandoc("""
			@echo off
			call "{}" amd64 -vcvars_ver={}
			if not "%ERRORLEVEL%"=="0" exit /b %ERRORLEVEL%
			cl
			""",).format(self.vcvarsall,self._vcvars_ver)
		with tempfile.NamedTemporaryFile('w',suffix='.cmd',delete=False)as tmp:
			try:
				tmp.write(contents)
				tmp.close()
				process=subprocess.Popen([tmp.name],stdout=subprocess.PIPE,stderr=subprocess.PIPE,universal_newlines=True,shell=False,)
				stdout,stderr=process.communicate()
				if process.returncode:
					return stdout.splitlines()+stderr.splitlines()
				return stderr.splitlines()
			finally:
				os.unlink(tmp.name)
def options(ctx):
	ctx.load('msvc')
	winsdk_default=re.findall(r'(8\.1|10(?:\.\d+)+)',os.environ.get('WINDOWSSDKVERSION',''),)or['']
	ctx.add_option('--winsdk-version',type=str,help='Windows SDK Versions, eg: 10.0.19041.0',default=winsdk_default[0],)
@contextlib.contextmanager
def _use_vs_toolsets_discovery():
	msvc=Context.load_tool('msvc')
	def gather_vswhere_versions(ctx,versions):
		msvc.gather_vswhere_versions(ctx,versions)
		log_debug=functools.partial(logging.getLogger('waflib').debug,extra={'zone':'msvc_toolset'},)
		for toolset in _VisualStudioToolset.discover(log_debug,ctx.cmd_and_log):
			waf_version='msvc {}'.format(toolset.visual_studio_version)
			if waf_version in versions:
				log_debug("Replacing msvc's '%s' with msvc_toolset's '%s'",waf_version,toolset)
			versions[waf_version]={target:msvc.target_compiler(ctx,'msvc',realtarget,toolset.visual_studio_version,'{} {} -vcvars_ver={}'.format(target,getattr(Options.options,'winsdk_version','')or ctx.env.WINSDK_VERSION or'',toolset.vc_version,),toolset.vcvarsall,)for target,realtarget in msvc.all_msvc_platforms[::-1]}
	try:
		Configure.conf(gather_vswhere_versions)
		yield
	finally:
		Configure.conf(msvc.gather_vswhere_versions)
def configure(ctx):
	with _use_vs_toolsets_discovery():
		ctx.load('msvc')
if __name__=='__main__':
	logger=logging.getLogger('msvc_toolset')
	logging.basicConfig(format='%(message)s',level=getattr(logging,os.environ.get('LOG_LEVEL','info').upper()),)
	for toolset in _VisualStudioToolset.discover(logger.debug):
		logger.info(toolset)
		cl_info='\n    '.join(line for line in toolset.cl()if line)
		logger.info('    %s',cl_info)
